"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeployFiles = createDeployFiles;
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const normalize_path_1 = __importDefault(require("normalize-path"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const constants_1 = require("@pnpm/constants");
const dp = __importStar(require("@pnpm/dependency-path"));
const DEPENDENCIES_FIELD = ['dependencies', 'devDependencies', 'optionalDependencies'];
function createDeployFiles({ allProjects, deployDir, lockfile, lockfileDir, rootProjectManifest, selectedProjectManifest, projectId, rootProjectManifestDir, }) {
    const deployedProjectRealPath = path_1.default.resolve(lockfileDir, projectId);
    const inputSnapshot = lockfile.importers[projectId];
    const targetSnapshot = {
        ...inputSnapshot,
        specifiers: {},
        dependencies: {},
        devDependencies: {},
        optionalDependencies: {},
    };
    const targetPackageSnapshots = {};
    for (const name in lockfile.packages) {
        const inputDepPath = name;
        const inputSnapshot = lockfile.packages[inputDepPath];
        const resolveResult = resolveLinkOrFile(inputDepPath, {
            lockfileDir,
            projectRootDirRealPath: rootProjectManifestDir,
        });
        const outputDepPath = resolveResult
            ? createFileUrlDepPath(resolveResult, allProjects)
            : inputDepPath;
        targetPackageSnapshots[outputDepPath] = convertPackageSnapshot(inputSnapshot, {
            allProjects,
            deployDir,
            deployedProjectRealPath,
            lockfileDir,
            projectRootDirRealPath: rootProjectManifestDir,
        });
    }
    for (const importerPath in lockfile.importers) {
        if (importerPath === projectId)
            continue;
        const projectSnapshot = lockfile.importers[importerPath];
        const projectRootDirRealPath = path_1.default.resolve(lockfileDir, importerPath);
        const packageSnapshot = convertProjectSnapshotToPackageSnapshot(projectSnapshot, {
            allProjects,
            deployDir,
            lockfileDir,
            deployedProjectRealPath,
            projectRootDirRealPath,
        });
        const depPath = createFileUrlDepPath({ resolvedPath: projectRootDirRealPath }, allProjects);
        targetPackageSnapshots[depPath] = packageSnapshot;
    }
    for (const field of DEPENDENCIES_FIELD) {
        const targetDependencies = targetSnapshot[field] ?? {};
        const targetSpecifiers = targetSnapshot.specifiers;
        const inputDependencies = inputSnapshot[field] ?? {};
        for (const name in inputDependencies) {
            const version = inputDependencies[name];
            const resolveResult = resolveLinkOrFile(version, {
                lockfileDir,
                projectRootDirRealPath: path_1.default.resolve(lockfileDir, projectId),
            });
            if (!resolveResult) {
                targetSpecifiers[name] = targetDependencies[name] = version;
                continue;
            }
            targetSpecifiers[name] = targetDependencies[name] =
                resolveResult.resolvedPath === deployedProjectRealPath ? 'link:.' : createFileUrlDepPath(resolveResult, allProjects);
        }
    }
    const result = {
        lockfile: {
            ...lockfile,
            overrides: undefined, // the effects of package overrides should already be part of the package snapshots
            patchedDependencies: undefined,
            packageExtensionsChecksum: undefined, // the effects of the package extensions should already be part of the package snapshots
            pnpmfileChecksum: undefined, // the effects of the pnpmfile should already be part of the package snapshots
            importers: {
                ['.']: targetSnapshot,
            },
            packages: targetPackageSnapshots,
        },
        manifest: {
            ...selectedProjectManifest,
            dependencies: targetSnapshot.dependencies,
            devDependencies: targetSnapshot.devDependencies,
            optionalDependencies: targetSnapshot.optionalDependencies,
            pnpm: {
                ...rootProjectManifest?.pnpm,
                ...(0, pick_1.default)(constants_1.USEFUL_NON_ROOT_PNPM_FIELDS, selectedProjectManifest.pnpm ?? {}),
                overrides: undefined, // the effects of package overrides should already be part of the package snapshots
                patchedDependencies: undefined,
                packageExtensions: undefined, // the effects of the package extensions should already be part of the package snapshots
            },
        },
    };
    if (lockfile.patchedDependencies) {
        result.lockfile.patchedDependencies = {};
        result.manifest.pnpm.patchedDependencies = {};
        for (const name in lockfile.patchedDependencies) {
            const patchInfo = lockfile.patchedDependencies[name];
            const resolvedPath = path_1.default.resolve(rootProjectManifestDir, patchInfo.path);
            const relativePath = (0, normalize_path_1.default)(path_1.default.relative(deployDir, resolvedPath));
            result.manifest.pnpm.patchedDependencies[name] = relativePath;
            result.lockfile.patchedDependencies[name] = {
                hash: patchInfo.hash,
                path: relativePath,
            };
        }
    }
    return result;
}
function convertPackageSnapshot(inputSnapshot, opts) {
    const inputResolution = inputSnapshot.resolution;
    let outputResolution;
    if ('integrity' in inputResolution) {
        outputResolution = inputResolution;
    }
    else if ('tarball' in inputResolution) {
        outputResolution = { ...inputResolution };
        if (inputResolution.tarball.startsWith('file:')) {
            const inputPath = inputResolution.tarball.slice('file:'.length);
            const resolvedPath = path_1.default.resolve(opts.lockfileDir, inputPath);
            const outputPath = (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, resolvedPath));
            outputResolution.tarball = `file:${outputPath}`;
            if (inputResolution.path)
                outputResolution.path = outputPath;
        }
    }
    else if (inputResolution.type === 'directory') {
        const resolvedPath = path_1.default.resolve(opts.lockfileDir, inputResolution.directory);
        const directory = (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, resolvedPath));
        outputResolution = { ...inputResolution, directory };
    }
    else if (inputResolution.type === 'git') {
        outputResolution = inputResolution;
    }
    else {
        const resolution = inputResolution; // `never` is the type guard to force fixing this code when adding new type of resolution
        throw new Error(`Unknown resolution type: ${JSON.stringify(resolution)}`);
    }
    return {
        ...inputSnapshot,
        resolution: outputResolution,
        dependencies: convertResolvedDependencies(inputSnapshot.dependencies, opts),
        optionalDependencies: convertResolvedDependencies(inputSnapshot.optionalDependencies, opts),
    };
}
function convertProjectSnapshotToPackageSnapshot(projectSnapshot, opts) {
    const resolution = {
        type: 'directory',
        directory: (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, opts.projectRootDirRealPath)),
    };
    const dependencies = convertResolvedDependencies(projectSnapshot.dependencies, opts);
    const optionalDependencies = convertResolvedDependencies(projectSnapshot.optionalDependencies, opts);
    return {
        dependencies,
        optionalDependencies,
        resolution,
    };
}
function convertResolvedDependencies(input, opts) {
    if (!input)
        return undefined;
    const output = {};
    for (const key in input) {
        const version = input[key];
        const resolveResult = resolveLinkOrFile(version, opts);
        if (!resolveResult) {
            output[key] = version;
            continue;
        }
        if (resolveResult.resolvedPath === opts.deployedProjectRealPath) {
            output[key] = 'link:.'; // the path is relative to the lockfile dir, which means '.' would reference the deploy dir
            continue;
        }
        output[key] = createFileUrlDepPath(resolveResult, opts.allProjects);
    }
    return output;
}
function resolveLinkOrFile(pkgVer, opts) {
    const { lockfileDir, projectRootDirRealPath } = opts;
    function resolveScheme(scheme, base) {
        if (!pkgVer.startsWith(scheme))
            return undefined;
        const { id, peersSuffix: suffix } = dp.parseDepPath(pkgVer.slice(scheme.length));
        const resolvedPath = path_1.default.resolve(base, id);
        return { scheme, resolvedPath, suffix };
    }
    const resolveSchemeResult = resolveScheme('file:', lockfileDir) ?? resolveScheme('link:', projectRootDirRealPath);
    if (resolveSchemeResult)
        return resolveSchemeResult;
    const { nonSemverVersion, patchHash, peersSuffix, version } = dp.parse(pkgVer);
    if (!nonSemverVersion)
        return undefined;
    if (version) {
        throw new Error(`Something goes wrong, version should be undefined but isn't: ${version}`);
    }
    const parseResult = resolveLinkOrFile(nonSemverVersion, opts);
    if (!parseResult)
        return undefined;
    if (parseResult.suffix) {
        throw new Error(`Something goes wrong, suffix should be undefined but isn't: ${parseResult.suffix}`);
    }
    parseResult.suffix = `${patchHash ?? ''}${peersSuffix ?? ''}`;
    return parseResult;
}
function createFileUrlDepPath({ resolvedPath, suffix }, allProjects) {
    const depFileUrl = url_1.default.pathToFileURL(resolvedPath).toString();
    const project = allProjects.find(project => project.rootDirRealPath === resolvedPath);
    const name = project?.manifest.name ?? path_1.default.basename(resolvedPath);
    return `${name}@${depFileUrl}${suffix ?? ''}`;
}
//# sourceMappingURL=createDeployFiles.js.map